import type {Maybe} from "./types.ts";

export abstract class Unit {
    public abstract up(): Promise<void>
    public abstract down(): Promise<void>
}

export class UnitManager {
    private state: 'init' | 'starting' | 'running' | 'stopping' | 'stopped' = 'init'
    private pendingUnits: Unit[] = []
    private runningUnits: Unit[] = []

    public async addUnit(unit: Unit): Promise<void> {
        this.pendingUnits.push(unit)
    }

    public getState(): typeof this.state {
        return this.state
    }

    public async run(): Promise<void> {
        await this.start()
        console.log('Press Ctrl-C to stop.')
        await new Promise<void>(res => {
            process.on('SIGINT', () => res())
        })
        await this.stop()
    }

    public async start(): Promise<void> {
        this.state = 'starting'

        let unit: Maybe<Unit>
        while ( unit = this.pendingUnits.shift() ) {
            try {
                await unit.up()
            } catch (e) {
                console.error('Error when starting unit')
                console.error(e)
                await this.stop()
                throw e
            }

            this.runningUnits.push(unit)
        }

        this.state = 'running'
    }

    public async stop(): Promise<void> {
        this.state = 'stopping'

        let unit: Maybe<Unit>
        while ( unit = this.runningUnits.shift() ) {
            try {
                await unit.down()
            } catch (e) {
                console.error('Error while stopping:')
                console.error(e)
            }
        }

        this.state = 'stopped'
    }
}
