import {Unit} from "../bones/Unit.ts";
import type {Maybe} from "../bones";
import {config} from "../config.ts";

export class HttpServer extends Unit {
    private server: Maybe<ReturnType<typeof Bun.serve>>

    async up(): Promise<void> {
        this.server = Bun.serve({
            hostname: config.http.address,
            port: config.http.port,
            fetch: async request => {
                const path = `${config.dirs.data}${new URL(request.url).pathname}`
                const file = Bun.file(path)
                return new Response(file)
            },
            error: () => new Response(
                JSON.stringify({ success: false, message: 'Not found.' }),
                {
                    status: 404,
                    headers: {
                        'Content-Type': 'application/json',
                    },
                }
            )
        })
        console.log(`Listening on ${config.http.address}:${config.http.port}`)
    }

    async down(): Promise<void> {
        console.log('Stopping server...')
        await this.server?.stop?.(true)
    }
}
