import {Unit} from "../bones/Unit.ts";
import type {Maybe} from "../bones";
import {undefined} from "zod";
import {refreshThreadsEntirely} from "../threads/refresh.ts";
import {config} from "../config.ts";

export class ThreadRefresher extends Unit {
    private handle: Maybe<Timer>
    private running: boolean = false

    async up(): Promise<void> {
        this.handle = setInterval(async () => {
            if ( this.running ) {
                return;
            }

            this.running = true
            try {
                console.log('Refreshing threads...')
                await refreshThreadsEntirely()
            } catch (e) {
                this.running = false
                throw e
            }

            console.log('Done.')
            this.running = false
        }, config.mail.refreshIntervalInSeconds * 1000)
    }

    async down(): Promise<void> {
        if ( this.handle ) {
            console.log('Stopping refresh interval...')
            clearInterval(this.handle)
            console.log('Done.')
        }
    }
}
